@@ -80,7 +80,7 @@ describe MessagePack::Factory do
       subject.register_type(0x00, Symbol)
       subject.freeze
       expect(subject.frozen?).to be_truthy
-      expect{ subject.register_type(0x01, Array) }.to raise_error(RuntimeError, "can't modify frozen Factory")
+      expect{ subject.register_type(0x01, Array) }.to raise_error(FrozenError, "can't modify frozen MessagePack::Factory")
     end
   end
 
@@ -249,6 +249,32 @@ describe MessagePack::Factory do
       my.b.should == 2
     end
 
+    describe "registering multiple ext type for the same class" do
+      let(:payload) do
+        factory = MessagePack::Factory.new
+        factory.register_type(0x01, Symbol, packer: :to_s, unpacker: :to_sym.to_proc)
+        factory.dump(:foobar)
+      end
+
+      it "select the type based on code for deserialization" do
+        factory = MessagePack::Factory.new
+        factory.register_type(0x00, Symbol, packer: -> (value) { raise NotImplementedError }, unpacker:  -> (value) { raise NotImplementedError })
+        factory.register_type(0x01, Symbol, packer: :to_s, unpacker: :to_sym.to_proc)
+        factory.register_type(0x02, Symbol, packer: -> (value) { raise NotImplementedError }, unpacker:  -> (value) { raise NotImplementedError })
+
+        expect(factory.load(payload)).to be == :foobar
+      end
+
+      it "uses the last registered packer for serialization" do
+        factory = MessagePack::Factory.new
+        factory.register_type(0x00, Symbol, packer: -> (value) { raise NotImplementedError }, unpacker:  -> (value) { raise NotImplementedError })
+        factory.register_type(0x02, Symbol, packer: -> (value) { raise NotImplementedError }, unpacker:  -> (value) { raise NotImplementedError })
+        factory.register_type(0x01, Symbol, packer: :to_s, unpacker: :to_sym.to_proc)
+
+        expect(factory.dump(:foobar)).to be == payload
+      end
+    end
+
     describe "registering an ext type for a module" do
       before do
         mod = Module.new do
@@ -660,6 +686,44 @@ describe MessagePack::Factory do
       expect(pool.load(pool.dump(42))).to be == 42
     end
 
+    it 'responds to #packer with a block' do
+      pool = factory.pool(1)
+      payload = pool.packer do |packer|
+        packer.write(42)
+        packer.full_pack
+      end
+      expect(payload).to be == factory.dump(42)
+    end
+
+    it 'responds to #unpacker with a block' do
+      pool = factory.pool(1)
+      payload = factory.dump(42)
+
+      object = pool.unpacker do |unpacker|
+        unpacker.feed(payload)
+        unpacker.read
+      end
+      expect(object).to be == 42
+    end
+
+    it '#packer does not allow to register types' do
+      pool = factory.pool(1)
+      expect do
+        pool.packer do |packer|
+          packer.register_type(0x20, ::MyType) { "type" }
+        end
+      end.to raise_error(FrozenError, "can't modify frozen MessagePack::Packer")
+    end
+
+    it '#unpacker does not allow to register types' do
+      pool = factory.pool(1)
+      expect do
+        pool.unpacker do |unpacker|
+          unpacker.register_type(0x20, ::MyType) { "type" }
+        end
+      end.to raise_error(FrozenError, "can't modify frozen MessagePack::Unpacker")
+    end
+
     it 'types can be registered before the pool is created' do
       factory.register_type(0x00, Symbol)
       pool = factory.pool(1)
