@@ -664,6 +664,16 @@ describe MessagePack::Unpacker do
       string *= 256
       MessagePack.unpack(MessagePack.pack(string)).encoding.should == string.encoding
     end
+
+    it 'returns correct size for array16 (issue #127)' do
+      unpacker.feed("\xdc\x00\x01\x01")
+      unpacker.read_array_header.should == 1
+    end
+
+    it 'returns correct size for map16 (issue #127)' do
+      unpacker.feed("\xde\x00\x02\x01\x02\x03\x04")
+      unpacker.read_map_header.should == 2
+    end
   end
 
   context 'extensions' do
@@ -707,6 +717,18 @@ describe MessagePack::Unpacker do
         described_class.new(:freeze => true)
       end
 
+      if (-"test").equal?(-"test") # RUBY_VERSION >= "2.5"
+        it 'dedups strings' do
+          interned_str = -"test"
+          roundtrip = MessagePack.unpack(MessagePack.pack(interned_str), freeze: true)
+          expect(roundtrip).to be interned_str
+
+          interned_str = -""
+          roundtrip = MessagePack.unpack(MessagePack.pack(interned_str), freeze: true)
+          expect(roundtrip).to be interned_str
+        end
+      end
+
       it 'can freeze objects when using .unpack' do
         parsed_struct = MessagePack.unpack(buffer, freeze: true)
         parsed_struct.should == struct
@@ -844,4 +866,25 @@ describe MessagePack::Unpacker do
       end
     end
   end
+
+  it "doesn't allow #dup or #clone" do
+    expect(subject).to_not respond_to :dup
+    expect(subject).to_not respond_to :clone
+  end
+
+  it "doesn't crash when marking an uninitialized buffer" do
+    if RUBY_PLATFORM == "java"
+      pending("THe java extension is missing Unpacker#buffer https://github.com/msgpack/msgpack-ruby/issues/315")
+    end
+
+    stress = GC.stress
+    begin
+      GC.stress = true
+
+      MessagePack::Unpacker.new.buffer
+      Object.new
+    ensure
+      GC.stress = stress
+    end
+  end
 end
